import _ from 'lodash';
import configureStore from 'redux-mock-store';
import {hoc} from '../';
import {mount} from 'enzyme';
import thunk from 'redux-thunk';

describe('Batch HOC', () => {
	let
		store;

	beforeEach(() => { store = configureStore([thunk])({}); });

	it('syncs remote data with redux store being applied with [fetchOnMount === true]', () => {
		mount(hoc.batch({
			baseURL     : '/',
			entity      : 'batch',
			fetchOnMount: true,
			key         : 'batch',
			requests    : () => [{}],
		})(() => null)({store}));

		expect(store.getActions().pop()).toEqual({
			request: {pathvars: null, params: {body: '[{}]', method: 'POST'}},
			syncing: false,
			type   : '@@redux-api@batch',
		});
	});

	it('doesn\'t sync remote data with redux store being applied with [fetchOnMount === false]', () => {
		mount(hoc.batch({
			baseURL     : '/',
			entity      : 'batch',
			fetchOnMount: false,
			key         : 'batch',
			requests    : () => [{}],
		})(() => null)({store}));

		expect(store.getActions().pop()).toEqual(undefined);
	});

	it('syncs remote data on [requests] change', () => {
		let
			instance = mount(hoc.batch({
				baseURL     : '/',
				entity      : 'batch',
				fetchOnMount: true,
				key         : 'batch',
				requests    : props => [{a: props.testProp}],
			})(() => null)({store, testProp: 'a'}));

		instance.setProps({store, testProp: 'b'});

		expect(store.getActions().pop()).toEqual({
			request: {pathvars: null, params: {body: '[{"a":"b"}]', method: 'POST'}},
			syncing: false,
			type   : '@@redux-api@batch',
		});
	});
});
