// @flow
import _ from 'lodash';
import base from './base';
import {compose, lifecycle} from 'recompose';
import {connect} from 'react-redux';
import {default as rest} from '../rest';
import syncOnMount from './sync-on-mount';

export default (hocParams: {
	baseURL?: string,
	entity: string | (props: Object) => string,
	fetchOnMount?: boolean,
	key: string | (props: Object) => string,
	queryParams?: (props: Object) => Object,
	urlParams?: (props: Object) => Object,
}) => (WrappedComponent: any) => compose(
	base(hocParams),

	connect(
		(state, props: {
			dispatch: Function,
			faFetched: Object,
			keyStatePath: string,
			resolvedEntity: string,
			resolvedKey: string,
		}) => ({
			faFetched: {
				...props.faFetched,

				[props.resolvedKey]: {
					...props.faFetched[props.resolvedKey],
					etag: _.get(state, `${props.keyStatePath}.lastResponse.headers.etag`),

					// Pick several methods from redux-api and apply dispatch to them. Can't be done in second param of connect()
					// as result objects of forst and second params of connect() are not properly merged
					..._(rest({
						baseURL: props.faFetched[props.resolvedKey].baseURL,
						entity : props.resolvedEntity,
						key    : props.resolvedKey,
					}).actions[props.resolvedEntity])

						.pick(['delete', 'post', 'put'])

						.map((value, key) => [
							key,
							(urlParams: {}, params?: {}, callback?: () => void) => props.dispatch(value(urlParams, params, callback)),
						])

						.fromPairs()
						.value(),
				},
			},
		}),
	),

	lifecycle({
		componentWillReceiveProps(nextProps) {
			if(nextProps.hasPropsChanged({nextProps, props: this.props, trackProps: ['queryParams', 'urlParams']}))
				nextProps.faFetched[nextProps.resolvedKey].sync(_.extend(
					{},
					nextProps.faFetched[nextProps.resolvedKey].urlParams,
					nextProps.getQueryParams(nextProps),
				));
		},
	}),

	syncOnMount(hocParams.fetchOnMount),
)(WrappedComponent);
