// @flow
import _ from 'lodash';
import reduxApi from 'redux-api';
import 'whatwg-fetch';
import entities from './entities';
import {withOptimistic} from './helpers';

export default (params: {
	baseURL: string,
	entity: string,
	key: string,
}) => reduxApi({
	[params.key]: withOptimistic(params.key, entities(params.baseURL)[params.entity]),
})
	.use('options', (url, params, getState) => {
		const
			token = _.get(getState(), 'auth.currentUser.access_token');

		return {
			headers: {
				Accept        : 'application/json',
				'Content-Type': 'application/json',
				...(token ? {Authorization: `Bearer ${token}`} : {}),
			},
		};
	})

	.use('fetch', (
		fetch => (url, opts) => fetch(url, opts).then(
			(response: Response) => response.status !== 204
				? response.json().then(J => ({data: J, headers: _.fromPairs([...response.headers.entries()])}))
				: {},
		)
	)(fetch));
